// -*- mode: c++; coding: cp1251 -*-
//
//    
//
// (c) Poroh Dmitry, 2003
//

#include "stdafx.h"
#include "log.h"
//
//
// Logger
//
//
Logger::Logger(const std::string& filename, Loglevel l)
    : ll(l)
{
    ofs.open(filename.c_str(),std::ios::out | std::ios::app);
	ofs << "LOG:   . =" << ll << std::endl;
    fs << "LOG:   . =" << ll << std::endl;
}

Logger::~Logger()
{
    fs << "LOG:   ." << std::endl;
    ofs << "LOG:   ." << std::endl;
    ofs.close();
}


void Logger::started(int n, int state)
{
    if (ll>=LL_STARTEND) {
        if (!by_num(n).loging())
            return;
        fs << "{  A" << n << "   : " << state
           << by_num(n).get_state(state) << " " << by_num(n) << std::endl;
        fs.flush();
        ofs << "{  A" << n << "   : " << state
            << " " << by_num(n).get_state(state) << " " << by_num(n) << std::endl;
        ofs.flush();
    }
}


void Logger::finished(int n, int state)
{
    if (ll>=LL_STARTEND) {
        if (!by_num(n).loging())
            return;
        fs << "}  A" << n << "    : " << state
           << " " <<  by_num(n).get_state(state) << " " << by_num(n) << std::endl;
        fs.flush();
        ofs << "}  A" << n << "    : " << state
            << " " <<  by_num(n).get_state(state) << " "  << by_num(n) << std::endl;
        ofs.flush();
    }
}

void Logger::inval_state(int n, int state)
{
    fs << "ERROR:  A" << n << "  : " << state
       << std::endl;
    fs.flush();
    ofs << "ERROR:  A" << n << "  : " << state
       << std::endl;
    ofs.flush();
}

void Logger::start_logging(int n)
{
    if (!by_num(n).loging()) {
        by_num(n).set_loging(true);
        fs << "LOG:     A" << n << " " << std::endl;
        fs.flush();
        ofs << "LOG:     A" << n << " " << std::endl;
        ofs.flush();
    }
}

void Logger::stop_logging(int n)
{
    if (by_num(n).loging()) {
        by_num(n).set_loging(false);
        fs << "LOG:     A" << n << " " << std::endl;
        fs.flush();
        ofs << "LOG:     A" << n << " " << std::endl;
        ofs.flush();
    }
}

void Logger::action(int n, std::string s )
{
    if (ll>=LL_ACTIONS) {
        fs << "* " << "z" << n << ": " << s << std::endl;
        fs.flush();
        ofs << "* " << "z" << n << ": " << s << std::endl;
        ofs.flush();
    }
}

void Logger::loglevel(Loglevel l)
{
    if (ll!=l) {
        fs << "LOG: " << "  : " << l << std::endl;
        ofs << "LOG: " << "  : " << l << std::endl;
        ll = l;
    }
}

Auto& Logger::by_num(int n)
{
    for (std::list<Auto>::iterator ci=a.begin(); ci!=a.end(); ++ci)
        if (ci->num()==n)
            return *ci;
    assert(0);
    exit(0);
}


//
//
// Auto
//
//

void Auto::show_var(const std::string& name, bool s)
{
    for (std::list<Automate_var>::iterator ci=vars.begin();ci!=vars.end();++ci)
        if (ci->getname()==name)
            ci->set_show(s);
}

bool Auto::show_var(const std::string& name) const
{
    for (std::list<Automate_var>::const_iterator ci=vars.begin();ci!=vars.end();++ci)
        if (ci->getname()==name)
            return ci->get_show();
    return false;
}


const State& Auto::get_state(int num)
{
    for (std::list<State>::const_iterator ci=states.begin();ci!=states.end();++ci)
        if (ci->num()==num)
            return *ci;
    assert(0);
}


//
//
// Streams
//
//
std::ostream& operator << (std::ostream& os, const Automate_var&s)
{
    if (!s.show)
        return os;
    os << s.name << "=";
    switch (s.type) {
    case VT_BOOLEAN:
        os << *((bool*)s.data);
        break;
    case VT_CHAR:
        os << '\'' << *((char*)s.data) << '\'';
        break;
    case VT_INTEGER:
        os << *((int*)s.data);
        break;
    }
    return os;
}

std::ostream& operator << (std::ostream&os, const Auto&a)
{
    for (std::list<Automate_var>::const_iterator ci=a.vars.begin();ci!=a.vars.end();++ci )
        os << *ci << " ";
    return os;
}

std::ostream& operator << (std::ostream&os, const State&s)
{
    os << "[ " << s.name << " ]";
	return os;
}
