// -*- mode: c++; coding: cp1251 -*-
//
//    
//
// (c) Poroh Dmitry, 2003
//
#ifndef _LOG_H_
#define _LOG_H_

#include <ostream.h>
#include <fstream>
#include <sstream>
#include <list>
#include <assert.h>

enum Loglevel {
    LL_SILENT,
    LL_STARTEND,
    LL_ACTIONS
};


enum VarType {
    VT_BOOLEAN,
    VT_INTEGER,
    VT_CHAR,
};

class Automate_var {
public:
    Automate_var(const std::string& name, char *,bool show=true);
    Automate_var(const std::string& name, int *,bool show=true);
    Automate_var(const std::string& name, bool *,bool show=true);
    void set_show(bool);
    bool get_show() const;
    std::string getname() const;
private:
    bool show;
    VarType type;
    void* data;
    std::string name;
    friend std::ostream& operator << (std::ostream&, const Automate_var&);
};

std::ostream& operator << (std::ostream&, const Automate_var&);

class State {
public:
    State(int, const std::string&);
    int num() const;
private:
    int number;
    std::string name;
    friend std::ostream& operator << (std::ostream&, const State&);
};

std::ostream& operator << (std::ostream&, const State&);

class Auto {
public:
    explicit Auto(int,bool logged=true);
    void add(const Automate_var&);
    void add(const State& );
    int num() const;
    bool loging() const;
    void set_loging(bool) const;
    void show_var(const std::string&, bool );
    bool show_var(const std::string&) const;
    const State& get_state(int num);
private:
    int number;
    mutable bool log;
    std::list<Automate_var> vars;
    std::list<State> states;
    friend std::ostream& operator << (std::ostream&, const Auto&);
};

std::ostream& operator << (std::ostream&, const Auto&);

class Logger {
public:
    Logger(const std::string& filename,Loglevel ll=LL_STARTEND);
    ~Logger();
    void add(Auto&);
    void started(int n, int state);
    void finished(int n, int state);
    void action(int n, std::string );
    void inval_state(int n, int state);
    void start_logging(int n);
    void stop_logging(int n);
    void loglevel(Loglevel ll);
    std::stringstream& stream();
    Auto& by_num(int n);
private:
    std::stringstream fs;
    std::ofstream ofs;
    std::list<Auto> a;
    int ll;
};

//
// inlines
//

inline Automate_var::Automate_var(const std::string& n,char *c,bool s)
    : show(s), name(n), data(c), type(VT_CHAR)
{}
inline Automate_var::Automate_var(const std::string& n,int *i,bool s)
    : show(s), name(n), data(i), type(VT_INTEGER)
{}
inline Automate_var::Automate_var(const std::string& n,bool *b,bool s)
    : show(s), name(n), data(b), type(VT_BOOLEAN)
{}
inline void Automate_var::set_show(bool s) { show = s; }
inline bool Automate_var::get_show() const { return show; }
inline std::string Automate_var::getname() const { return name; }

inline State::State(int num, const std::string& state_name)
    : number(num), name(state_name)
{}
inline int State::num() const { return number; }

inline Auto::Auto(int n, bool l)
    : number(n), log(l)
{}
inline void Auto::add(const Automate_var&v) { vars.push_back(v); }
inline void Auto::add(const State&s ) { states.push_back(s); }
inline int Auto::num() const { return number; }
inline bool Auto::loging() const { return log; }
inline void Auto::set_loging(bool l) const { log = l; }
inline void Logger::add(Auto&at) { a.push_back(at); }
inline std::stringstream& Logger::stream() { return fs; }


#endif
